@@ -67,6 +67,8 @@ def filter_settings_by_register(obj,param):
         "file_bak":"Scrapy_Prj.pipelines.ossfilepipelines.OssFilesPipelineBak",
         "img":"Scrapy_Prj.pipelines.ossimgpipelines.OssImagesPipeline",
         "img_bak":"Scrapy_Prj.pipelines.ossimgpipelines.OssImagesPipelineBak",
+        "other_file": "Scrapy_Prj.pipelines.ossotherfilepipelines.OssOtherFilesPipeline",
+        "other_file_bak": "Scrapy_Prj.pipelines.ossotherfilepipelines.OssOtherFilesPipelineBak",
         "pyppeteer":"gerapy_pyppeteer.downloadermiddlewares.PyppeteerMiddleware",
         "bloom":"Scrapy_Prj.pipelines.bloomerfiler.BloomFilterPipeline",
         "mysql":"Scrapy_Prj.pipelines.mysqlpipelines.MySQLPipeline",
