@@ -7,11 +7,11 @@
 选择目录位置:{python}/Lib/site-packages/scrapy/xcc_pipelines
 """
 
+from w3lib.url import safe_url_string
 import hashlib
 import logging
 import os
 from contextlib import suppress
-from io import BytesIO
 from urllib.parse import urlparse
 
 from itemadapter import ItemAdapter
@@ -32,18 +32,23 @@ import requests
 
 import pdfplumber
 from PyPDF2 import PdfFileReader, PdfFileWriter
-from PyPDF2.utils import PdfReadError
-import io,re
+import fitz,img2pdf
+
+import io
 from scrapy.http.headers import Headers
 from scrapy.utils.conf import get_config
 from twisted.internet.defer import DeferredList
+from datetime import datetime
 
 logging.getLogger('pdfplumber').setLevel(logging.WARNING)
 logging.getLogger('PyPDF2').setLevel(logging.WARNING)
 logging.getLogger('pdfminer').setLevel(logging.WARNING)
 logging.getLogger('urllib3').setLevel(logging.WARNING)
 logging.getLogger('oss2').setLevel(logging.WARNING)
+logging.getLogger('img2pdf').setLevel(logging.WARNING)
+logging.getLogger('PIL').setLevel(logging.WARNING)
 logger = logging.getLogger(__name__)
+
 class OSSFilesStore:
     def __init__(self,uri):
         if not uri.startswith("https://xcc2.oss"):
@@ -57,7 +62,10 @@ class OSSFilesStore:
             relative_path = path.replace(f"{self.basedir}/","")
             content= buf.getvalue()
             body = self.del_dirty_page(content=content,according_to=self.according_to,check_num=self.check_num) if self.deal_with_pdf else content
-            self._bucket.put_object(relative_path,body, progress_callback=None)
+            pushobj_result = self._bucket.put_object(relative_path,body, progress_callback=None)
+            if pushobj_result.status!=200:
+                raise FileException('Push-obj-error')
+
 
     def del_dirty_page(self,content,according_to,check_num):
         '''根据关键词剔除pdf匹配页
@@ -163,16 +171,16 @@ class OssFilesPipeline(MediaPipeline):
         self.filter_empty_item = settings.get(
             'FILTER_EMPTY_ITEM', self.FILTER_EMPTY_ITEM
         )
-        self.invalid_fild_path = settings.get(
+        self.invalid_fild_paths = settings.get(
             #invalid_file_body
-            'FILTER_INVALID_FILE_PATH', False
+            'FILTER_INVALID_FILE_PATHS', []
         )
         self.files_oss_field = settings.get('FILE_PIPE_CONFIG',{}).get(self.__class__.__name__,{}).get('FILES_OSS_URLS_FIELD', self.DEFAULT_FILES_OSS_URLS_FIELD)
         self.files_oss_result_field = settings.get('FILE_PIPE_CONFIG',{}).get(self.__class__.__name__,{}).get('FILES_OSS_RESULT_FIELD', self.DEFAULT_FILES_OSS_RESULT_FIELD)
         self.page_logo_text = settings.get('FILE_PIPE_CONFIG',{}).get(self.__class__.__name__,{}).get('PAGE_LOGO_TEXT','')
         self.item_header_dict = settings.get('FILE_PIPE_CONFIG',{}).get(self.__class__.__name__,{}).get('ITEM_HEADER',None)
-        self.files_oss_file = "items/"+settings.attributes.get("OSS_FILE")
-        self.if_feed = settings.get('FEEDS').attributes 
+        self.files_oss_file = "/".join(["items",datetime.now().strftime("%Y%m"),settings.attributes.get("OSS_FILE")])
+        self.if_feed = settings.get('FEEDS').attributes
         self.file_pipe_config = file_pipe_config
         self.ext_model = settings.get('EXT_MODEL',False)
         super().__init__(download_func=download_func, settings=settings)
@@ -243,61 +251,42 @@ class OssFilesPipeline(MediaPipeline):
                 extra={'spider': info.spider}
             )
             raise FileException('download-error')
-        def check_eof_error(response):
-            # 判断是否pdf有eof结构
-            content = response.body
-            idx = content.rfind(rb'%%EOF')
-            partEnd = content[(0 if idx-100 < 0 else idx-100) : idx + 5]
-            if not re.search(rb'startxref\s+\d+\s+%%EOF$', partEnd):
-                print('Error: not find startxref')
-                try:
-                    content = content+b'%%EOF'
-                    response = response.replace(body=content)
-                    return response
-                except Exception as e:
-                    print(e)
-                    return None
-            else :
-                return response
 
         # response.body = content
         if magic.from_buffer(response.body, mime=True).split('/')[-1] == 'pdf':
-            for i in range(3):
+            retry_invalid_times = 3
+            for i in range(retry_invalid_times):
                 # 如果pdf损坏重试
                 try:
-                    response = check_eof_error(response)
-                    reader = PdfFileReader(io.BytesIO(response.body))
-                    if reader.getNumPages() < 1:    #进一步通过页数判断。
-                        raise FileException('文件损坏,requests库重试')
-                    else:
-                        break
-                except PdfReadError as e:
-                    if "EOF marker not found" in str(e):
-                        break
-                    elif "not found" in str(e):
-                        print(response.request.headers)
-                        print(str(e))
-                        headers = Headers(response.request.headers).to_unicode_dict()
-                        contents = requests.get(response.request.url,headers = headers, timeout=(10,120)).content
-                        response = response.replace(body=contents)
-                        response = check_eof_error(response)
-                        logging.info(f"Retry {i+1} times , Url is {response.request.url}")
+                    valid = fitz.Document(stream=response.body, filetype="pdf")
+                    if valid.is_repaired:
+                        raise FileException('download-error')
+                    break
+                except Exception as e:
+                    headers = Headers(response.request.headers).to_unicode_dict()
+                    logging.warn(f"请求头: {headers} , PDF下载异常: {e}")
+                    contents = requests.get(response.request.url,headers = headers, timeout=(10,120)).content
+                    response = response.replace(body=contents)
+                    logging.info(f"Retry {i+1} times , Url is {response.request.url}")
+                    if i+1 >= retry_invalid_times : return
+                
+        media_ext_guss = os.path.splitext(os.path.basename(urlparse(request.url).path))[1]
+
+        if media_ext_guss==".pdf":
             try:
-                reader = PdfFileReader(io.BytesIO(response.body)) # 大部分损坏pdf在此read报错
-                if reader.getNumPages() < 1:    #进一步通过页数判断。
-                    raise FileException(f'文件损坏,{response.request.url}')
-            except PdfReadError as e:
-                if "EOF marker not found" in str(e):
-                    pass
-                elif "not found" in str(e):
-                    return
-                    
-        if self.invalid_fild_path:
-            # invalid_fild_path :: 失效文件路径,表示遇到该文件内容与invalid_fild_path路径下内容一致时返回空
-            with open(f"{self.invalid_fild_path}",'rb') as f:
-                invalid_file_body = f.read()
-            if invalid_file_body == response.body:
-                return {'url': request.url, 'path': ''}
+                valid = fitz.Document(stream=response.body, filetype="pdf")
+            except Exception as e:
+                print(e)
+                return
+
+        if self.invalid_fild_paths:
+            # invalid_fild_path <list> :: 失效文件路径,表示遇到该文件内容与invalid_fild_path路径下内容一致时返回空
+            invalid_file_body_list = list()
+            for path in self.invalid_fild_paths:
+                with open(f"{path}",'rb') as f:
+                    invalid_file_body_list.append(f.read())
+                if response.body in invalid_file_body_list:
+                    return {'url': request.url, 'path': ''}
 
         if not response.body:
             logger.warning(
@@ -339,29 +328,38 @@ class OssFilesPipeline(MediaPipeline):
 
         return {'url': request.url, 'path': path, 'checksum': checksum, 'status': status}
 
+    def transf_img2_pdf(self,img_urls):
+        # if "png" in img_urls or "jpg" in img_urls:
+        stream_list = list()
+        for img in img_urls.split('|'):
+            stream_list.append(requests.get(img,timeout=10).content)
+        path = ".".join([img.rsplit(".",1)[0],"pdf"])
+        buf = io.BytesIO(img2pdf.convert(stream_list))
+        self.store.persist_file(path,buf,info=None)
+        return path
+        
+
     def get_media_requests(self, item, info):
         # yield item 由此函数修改切割生成请求
         if not hasattr(item,'_refer'):
             item.__setattr__('_refer','')
         if not hasattr(item,'_header'):
             item.__setattr__('_header',{})
+        if not hasattr(item,'_set_cookies'):
+            item.__setattr__('_set_cookies',{})
         if self.file_pipe_config.get(self.__class__.__name__,{}).get("RESOURCE_CLASSNAME",'').lower() in item.__class__.__name__.lower():
             # TAG: 由于字段长度限制,限制字段urls个数100个以内
             urls = ItemAdapter(item).get(self.file_pipe_config.get(self.__class__.__name__,{}).get("FILES_OSS_URLS_FIELD",self.files_oss_field), '').split("|") \
                 if ItemAdapter(item).get(self.file_pipe_config.get(self.__class__.__name__,{}).get("FILES_OSS_URLS_FIELD",self.files_oss_field), '') else []
             def request_add_params(urls):
                 for u in urls:
-                    if '%' not in u:
-                        #防止资源链接出现空格等特殊字符,eg:https://www.semiee.com/file/Infineon/Infineon-TLE9261-3BQX V33.pdf
-                        # u = quote(u, safe='/:&?=#') 
-                        u = u.replace(" ","%20")
-                    u = Request(u)
+                    u = Request(safe_url_string(u))
                     # 打开爬虫中间件后,_header不为空,取资源链接上一请求的header和referer,
                     # 如果header中包含cookie或响应中包含set-cookie请配置COOKIE_ENABLE=False
-                    if '_refer' in item.__dict__ and '_header' not in item.__dict__ :
+                    if item._refer :
                         #爬虫文件中如果给item设置_refer属性在此添加
                         item._header['Referer']=item._refer
-                    if '_set_cookies' in item.__dict__:
+                    if item._set_cookies :
                         #爬虫文件中如果给item设置_set_cookies属性在此添加
                         item._header['Cookie']=item._set_cookies
                     u.headers = Headers(item._header, encoding='utf-8')
@@ -379,7 +377,7 @@ class OssFilesPipeline(MediaPipeline):
 
     def file_downloaded(self, response, request, info, *, item=None):
         path = item
-        buf = BytesIO(response.body)
+        buf = io.BytesIO(response.body)
         checksum = md5sum(buf)
         buf.seek(0)
         self.store.persist_file(path, buf, info)
@@ -388,8 +386,13 @@ class OssFilesPipeline(MediaPipeline):
     def item_completed(self, results, item, info):
         if self.file_pipe_config.get(self.__class__.__name__,{}).get("RESOURCE_CLASSNAME",'').lower() in item.__class__.__name__.lower():
             with suppress(TypeError):
-                if '|'.join([x['path'] for ok, x in results if ok and x['path']]):
-                    ItemAdapter(item)[self.file_pipe_config.get(self.__class__.__name__,{}).get('FILES_OSS_RESULT_FIELD',self.files_oss_result_field)] = '|'.join([x['path'] for ok, x in results if ok and x['path']])
+                img_urls = '|'.join([x['path'] for ok, x in results if ok])
+                if img_urls :
+                    if "png" in img_urls or "jpeg" in img_urls :
+                        obj_path = self.transf_img2_pdf(img_urls)
+                        ItemAdapter(item)[self.file_pipe_config.get(self.__class__.__name__,{}).get('FILES_OSS_RESULT_FIELD',self.files_oss_result_field)] = obj_path
+                    else:
+                        ItemAdapter(item)[self.file_pipe_config.get(self.__class__.__name__,{}).get('FILES_OSS_RESULT_FIELD',self.files_oss_result_field)] = '|'.join([x['path'] for ok, x in results if ok])
             # TAG: 如果决定资源字段返回为空不存表,修改 FILTER_EMPTY_ITEM = True , 默认false
             with suppress(KeyError):
                 if not item.get(self.file_pipe_config.get(self.__class__.__name__,{}).get("FILES_OSS_RESULT_FIELD",''),"") and self.filter_empty_item:
